/**********************************************************************
** Copyright (C) 2000 Trolltech AS.  All rights reserved.
**
** This file is part of Qtopia Environment.
**
** This file may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
** See http://www.trolltech.com/gpl/ for GPL licensing information.
**
** Contact info@trolltech.com if any conditions of this licensing are
** not clear to you.
**
**********************************************************************/

#include "clock.h"

#include <qpe/qpeapplication.h>
#include <qpe/qcopenvelope_qws.h>
#include <qpe/config.h>
#include <qpe/resource.h>
#include <qpe/applnk.h>
#include <qpe/timestring.h>

#include <qtimer.h>
#include <qpopupmenu.h>
#include <qpainter.h>
#include <qrect.h>
#include <qapplication.h>
	
#include "AlarmDetails.h"

LauncherClock::LauncherClock( QWidget *parent ) : QWidget( parent )
{
	m_dateTime = Resource::loadImage("DateTime").smoothScale(AppLnk::smallIconSize(),  AppLnk::smallIconSize());
	m_alarm = Resource::loadImage("qpealarmclock/smallalarm").smoothScale(AppLnk::smallIconSize(), AppLnk::smallIconSize());

	//setFont( QFont( "Helvetica", 9, QFont::Bold ) );
	setFont(QApplication::font());

    // If you want a sunken border around the clock do this:
    // setFrameStyle( QFrame::Panel | QFrame::Sunken );
    //setFont( QFont( "Helvetica", , QFont::Normal ) );
    connect( qApp, SIGNAL( timeChanged() ), this, SLOT( updateTime( ) ) );
    connect( qApp, SIGNAL( clockChanged( bool ) ), this, SLOT( slotClockChanged( bool ) ) );

    QCopChannel* chan = new QCopChannel("QPE/clockApplet", this );
    connect(chan, SIGNAL(received(const QCString&,const QByteArray&) ),
            this, SLOT(slotMessage(const QCString&, const QByteArray& ) ) );

    readConfig();
	loadAlarms();

    timerId = 0;
    timerEvent(0);
    show();
}

void LauncherClock::readConfig() 
{
    Config config( "qpe" );
    config.setGroup( "Time" );
    ampmFormat = config.readBoolEntry( "AMPM", TRUE );
    config.setGroup( "Date" );
    format = config.readNumEntry("ClockApplet",0);
	m_showAlarm = config.readBoolEntry("showAlarm", TRUE);
}

void LauncherClock::mousePressEvent( QMouseEvent * )
{
	QPopupMenu *menu = new QPopupMenu ( this );
	QString cmd;

	QDateTime dt = QDateTime::currentDateTime();
	QString fmt = "dddd, dd MMMM yyyy hh:mm";
	if(ampmFormat) 
		fmt += "AP";

	menu->insertItem(QIconSet(m_dateTime), TimeString::timeString(dt.time(), ampmFormat, false) + " " 
		+ TimeString::dateString(dt.date()), this, SLOT(slotSetDateClicked()));
	menu->insertSeparator();

	for (int i = 0; i < NUM_ALARMS; i++)
	{
		if (alarms[i] != 0)
		{
			if (alarms[i]->enabled())
			{
				QString label = alarmText(alarms[i]);
				int id = menu->insertItem (QIconSet(m_alarm), label, this, SLOT(slotEditAlarmClicked(int)));
				menu->setItemParameter(id, i);
				//menu->setItemChecked(id, alarms[i]->enabled());
			}
		}
	}

	menu->insertItem (tr("Edit Alarms..."), this, SLOT(slotEditAlarmsClicked()));

	QPoint p = mapToGlobal ( QPoint ( 0, 0 ) );
	QSize s = menu-> sizeHint ( );

	p = QPoint ( p. x ( ) + ( width ( ) / 2 ) - ( s. width ( ) / 2 ), p. y ( ) - s. height ( ));
	menu->exec(p);
	delete menu;
}

QString LauncherClock::alarmText(AlarmDetails* alarmDetails)
{
	QString txt;

    if (ampmFormat)
    {
      if (alarmDetails->time().hour() >= 12)
      {
        int hour = alarmDetails->time().hour() == 12 ? 12 :
                   alarmDetails->time().hour() - 12;

        txt += QString::number(hour) + ":" +
                         QString().sprintf("%02d",
                         alarmDetails->time().minute()) +
                         tr(" PM");
      }
      else
      {
        txt += QString::number(alarmDetails->time().hour() == 0 ?
                         12 : alarmDetails->time().hour()) +
                         ":" + QString().sprintf("%02d",
                         alarmDetails->time().minute()) +
                         tr(" AM");
      }
    }
    else
    {
      txt += QString().sprintf("%02d:%02d",
                       alarmDetails->time().hour(),
                       alarmDetails->time().minute());
    }

	QDateTime nextDt = alarmDetails->nextDateTime();

	txt += " " + TimeString::shortDate(nextDt.date());
	/*
	txt += " " + QString::number(nextDt.date().dayOfWeek());
	txt += ", " + QString::number(nextDt.date().day());
	*/

	txt += " - " + alarmDetails->message();

	const size_t MAX_LEN = 45;
    if (txt.length() > MAX_LEN)
    {
      txt.truncate(MAX_LEN);
      txt.append("...");
    }

    //txt += " (" + AlarmDetails::frequencyLabel(alarmDetails->frequency()) + ")";
	return txt;
}


void LauncherClock::loadAlarms()
{
    for (int i = 0; i < NUM_ALARMS; i++)
      alarms[i] = new AlarmDetails(i);
}

void LauncherClock::clearAlarms()
{
    for (int i = 0; i < NUM_ALARMS; i++)
      delete alarms[i];
}

bool LauncherClock::hasActiveAlarms()
{
	for (int i = 0; i < NUM_ALARMS; i++)
	{
		if (alarms[i] != 0 && alarms[i]->enabled())
			return true;
	}

	return false;
}


void LauncherClock::slotEditAlarmsClicked()
{
	QCopEnvelope e("QPE/Application/qpealarmclock", "showAlarms()");
}

void LauncherClock::slotEditAlarmClicked(int alarm)
{
	QCopEnvelope e("QPE/Application/qpealarmclock", "editAlarm(int)");
	e << alarm;
}

void LauncherClock::slotSetDateClicked()
{
	QCString setTimeApp;
	QCopEnvelope e("QPE/Application/systemtime", "setDocument(QString)");
	QString lf = QString::null;

	e << lf;
}

void LauncherClock::slotMessage(const QCString& msg, const QByteArray& /* data */)
{
    //int alarmNumber;
    //QDataStream ds(data, IO_ReadOnly);
	if (msg == "alarmsChanged()") 
	{
		reloadAlarms();
		update();
	}
}

void LauncherClock::timerEvent( QTimerEvent *e )
{
    if ( !e || e->timerId() == timerId ) 
	{
		killTimer( timerId );
		changeTime();
		QTime t = QTime::currentTime();
		int ms = (60 - t.second())*1000 - t.msec();
		timerId = startTimer( ms );
    } 
	else 
	{
		QWidget::timerEvent( e );
    }
}

void LauncherClock::updateTime( void )
{
    changeTime();
}

void LauncherClock::changeTime( void )
{
    QTime tm = QDateTime::currentDateTime().time();
    QString s;
    if( ampmFormat ) 
	{
		int hour = tm.hour();
		if (hour == 0)
			hour = 12;
		if (hour > 12)
			hour -= 12;
		s.sprintf( "%2d:%02d %s", hour, tm.minute(),
			(tm.hour() >= 12) ? "PM" : "AM" );
    } 
	else
	  s.sprintf( "%2d:%02d", tm.hour(), tm.minute() );

    if (format==1) 
	{
		QDate dm = QDate::currentDate();
		QString d;
		d.sprintf("%d/%d ", dm.day(), dm.month());
		setText( d+s );
    } 
	else if (format==2) 
	{
		QDate dm = QDate::currentDate();
		QString d;
		d.sprintf("%d/%d ", dm.month(), dm.day());
		setText( d+s );
    } 
	else 
	{
	  setText( s );
    }

	update();
}

void LauncherClock::paintEvent (QPaintEvent *)
{
	QPainter p (this);
	p.setFont( QApplication::font() );
	QRect rc = p.fontMetrics().boundingRect(m_text);

	setFixedHeight(rc.height());
	setFixedWidth (rc.width() + (hasActiveAlarms() ? m_alarm.width() : 0));

	int y, x = 0;
	if (hasActiveAlarms())
	{
		if (m_showAlarm)	// user may want to disable this
		{
			y = (height() - m_alarm.height()) / 2;
			p.drawPixmap(x, y, m_alarm);
			x += m_alarm.width();
		}
	}

	// calculate Y position (text centered in widget, offset to font baseline)
	y = ( (height() - rc.height()) / 2 ) + p.fontMetrics().ascent();
	p.drawText(x, y, m_text);
}

void LauncherClock::slotClockChanged( bool pm )
{
    readConfig();
    updateTime();
}
